// YouTube.js for PSPTube20080224 (by FreePlay)

function YouTube_CheckURL( url, option )
{
	// ex. http://www.youtube.com/watch?v=oHg5SJYRHA0
	if(url.match( /^http:\/\/www\.youtube\.com\/watch\?v=.*/ )) {
		return 1;
	} else {
		return 0;
	}
}

/*
 Get a YouTube URL
*/
function YouTube_GetURL( url, option )
{
	// Grab video ID:
	// "movie=" video ID
	var video_id = url.match( /^http:\/\/www\.youtube\.com\/watch\?v=(.*)/ );
	if(video_id == null) { return null; /* ID not valid */ }
	video_id = RegExp.$1;

	var contents = GetContents( "http://www.youtube.com/watch?v=" + video_id );
	if(contents == null) { return null; /* error getting page */ }

	// get video file URL
	var reg = /\/watch_fullscreen\?.+&video_id=([^&]+).*&t=([^&]+).*/.exec(contents);
	if(reg == null) { return null; } /* no match - invalid video watching page */
	var video_id = RegExp.$1;
	var token = RegExp.$2;
	var video_url = 'http://youtube.com/get_video?video_id='+video_id+'&t='+token;

	return video_url;
}

/*
 Video search
 
 keyword     : keyword to search
 start_index : starting index (based on 1)
 length      : number of videos to search for
 option      : The current search criteria, defaults to 0 (?)
 
 Return info
 
 .keyword                            : search string
 .total                              : total number of results
                                                -1 : error
                                                 0 : no results
                                                 >0: total results
 .start                              : starting number (based on 1)
 .end                                : ending number (based on 1)
 .VideoInfo                          : video sequence information
 .VideoInfo[n].Author                : Author (string)
 .VideoInfo[n].Title                 : Title (string)
 .VideoInfo[n].LengthSeconds         : LengthSeconds: total video playback time (in seconds) (integer)
 .VideoInfo[n].RatingAvg             : RatingAvg: rating (float)
 .VideoInfo[n].RatingCount           : RatingCount: number of votes (integer)
 .VideoInfo[n].Description           : Description (string)
 .VideoInfo[n].ViewCount             : ViewCount (integer)
 .VideoInfo[n].UploadTime            : UploadTime: Video upload time (UNIX style)
 .VideoInfo[n].CommentCount          : CommentCount (integer)
 .VideoInfo[n].MylistCount           : MylistCount: ? (integer)
 .VideoInfo[n].Tags                  : Tags: Space-delimited string of tags, e.g. "tag1 tag2 tag3"
 .VideoInfo[n].URL                   : URL: Link to viewing page (string)
 .VideoInfo[n].ThumbnailURL          : ThumbnailURL: URL to thumbnail (string)
 .VideoInfo[n].SaveFilename          : SaveFilename (string)
                                                "Movie deprecated set unique ID please."
 .VideoInfo [n]. Attr: Integer OR video attribute set to the following value
                                                 1: read-only (not delete)
                                                 2: download
                                                 4: Play is a need to connect to the network
                                                 Usually, 1 +2 +4, 7
                                                 MS is a directory, so one can not be removed
                                                 MS files, 0
 .VideoInfo[n].child				 : children
 .VideoInfo[n].parent				 : parents
*/
function YouTube_Search( keyword, start_index, length, option )
{
	var pattern = /<entry>((.|\r|\n)*?)<\/entry>/;
	var result = new Object();
    result.keyword   = keyword;		// Search
	result.VideoInfo = new Array();	// Results in sequence
	result.start     = start_index;	// Index of first result
	result.end       = 0;
	result.total     = -1;

	// API calls for searching
	var url_base = "http://gdata.youtube.com/feeds/api/videos?vq=" + PSPTube.encodeURI( keyword )+"&start-index="+start_index+"&max-results="+length;

	var nPage = Math.floor( (start_index - 1) / 24 ) + 1;	/* begin the search page */
	var nIndex = (nPage - 1) * 24 + 1;						/* begin index */
	while(nIndex < (start_index + length)) {
		var url = url_base;
		var contents = GetContents( url );
		//var contents="<?xml version='1.0' encoding='UTF-8'?><feed xmlns='http://www.w3.org/2005/Atom' xmlns:openSearch='http://a9.com/-/spec/opensearchrss/1.0/' xmlns:gml='http://www.opengis.net/gml' xmlns:georss='http://www.georss.org/georss' xmlns:media='http://search.yahoo.com/mrss/' xmlns:yt='http://gdata.youtube.com/schemas/2007' xmlns:gd='http://schemas.google.com/g/2005'><id>http://gdata.youtube.com/feeds/api/videos?start-index=1&amp;max-results=20&amp;vq=taco</id><updated>2008-02-24T23:34:53.068Z</updated><category scheme='http://schemas.google.com/g/2005#kind' term='http://gdata.youtube.com/schemas/2007#video'/><title type='text'>YouTube Videos matching query: taco</title><logo>http://www.youtube.com/img/pic_youtubelogo_123x63.gif</logo><link rel='alternate' type='text/html' href='http://www.youtube.com'/><link rel='http://schemas.google.com/g/2005#feed' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos'/><link rel='self' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos?start-index=1&amp;max-results=20&amp;vq=taco'/><link rel='next' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos?start-index=21&amp;max-results=20&amp;vq=taco'/><author><name>YouTube</name><uri>http://www.youtube.com/</uri></author><generator version='beta' uri='http://gdata.youtube.com/'>YouTube data API</generator><openSearch:totalResults>23509</openSearch:totalResults><openSearch:startIndex>1</openSearch:startIndex><openSearch:itemsPerPage>20</openSearch:itemsPerPage><entry><id>http://gdata.youtube.com/feeds/api/videos/OG3PnQ3tgzY</id><published>2007-05-17T14:21:39.000-07:00</published><updated>2007-05-17T14:21:39.000-07:00</updated><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Ockerse'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='on'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Ritz'/><category scheme='http://schemas.google.com/g/2005#kind' term='http://gdata.youtube.com/schemas/2007#video'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='80&apos;s'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='1984'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='the'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='After'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Puttin&apos;'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Taco'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Eighties'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Eight'/><category scheme='http://gdata.youtube.com/schemas/2007/categories.cat' term='Music' label='Music'/><title type='text'>Taco - Puttin' on the Ritz (Original Uncensored Version)</title><content type='text'>This is the original version of the video. The infamous &quot;blackface&quot; that was removed is shown here.</content><link rel='alternate' type='text/html' href='http://www.youtube.com/watch?v=OG3PnQ3tgzY'/><link rel='http://gdata.youtube.com/schemas/2007#video.responses' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/OG3PnQ3tgzY/responses'/><link rel='http://gdata.youtube.com/schemas/2007#video.related' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/OG3PnQ3tgzY/related'/><link rel='self' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/OG3PnQ3tgzY'/><author><name>ellocorod</name><uri>http://gdata.youtube.com/feeds/api/users/ellocorod</uri></author><media:group><media:title type='plain'>Taco - Puttin' on the Ritz (Original Uncensored Version)</media:title><media:description type='plain'>This is the original version of the video. The infamous &quot;blackface&quot; that was removed is shown here.</media:description><media:keywords>1984, 80's, After, Eight, Eighties, Ockerse, on, Puttin', Ritz, Taco, the</media:keywords><yt:duration seconds='273'/><media:category label='Music' scheme='http://gdata.youtube.com/schemas/2007/categories.cat'>Music</media:category><media:content url='http://www.youtube.com/v/OG3PnQ3tgzY' type='application/x-shockwave-flash' medium='video' isDefault='true' expression='full' duration='273' yt:format='5'/><media:content url='rtsp://rtsp2.youtube.com/ChoLENy73wIaEQk2g-0Nnc9tOBMYDSANFEgGDA==/0/0/0/video.3gp' type='video/3gpp' medium='video' expression='full' duration='273' yt:format='1'/><media:content url='rtsp://rtsp2.youtube.com/ChoLENy73wIaEQk2g-0Nnc9tOBMYESARFEgGDA==/0/0/0/video.3gp' type='video/3gpp' medium='video' expression='full' duration='273' yt:format='6'/><media:player url='http://www.youtube.com/watch?v=OG3PnQ3tgzY'/><media:thumbnail url='http://img.youtube.com/vi/OG3PnQ3tgzY/2.jpg' height='97' width='130' time='00:02:16.500'/><media:thumbnail url='http://img.youtube.com/vi/OG3PnQ3tgzY/1.jpg' height='97' width='130' time='00:01:08.250'/><media:thumbnail url='http://img.youtube.com/vi/OG3PnQ3tgzY/3.jpg' height='97' width='130' time='00:03:24.750'/><media:thumbnail url='http://img.youtube.com/vi/OG3PnQ3tgzY/0.jpg' height='240' width='320' time='00:02:16.500'/></media:group><yt:statistics viewCount='200446'/><gd:rating min='1' max='5' numRaters='786' average='4.74'/><gd:comments><gd:feedLink href='http://gdata.youtube.com/feeds/api/videos/OG3PnQ3tgzY/comments' countHint='0'/></gd:comments></entry><entry><id>http://gdata.youtube.com/feeds/api/videos/OG3PnQ3tgzY</id><published>2007-05-17T14:21:39.000-07:00</published><updated>2007-05-17T14:21:39.000-07:00</updated><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Ockerse'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='on'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Ritz'/><category scheme='http://schemas.google.com/g/2005#kind' term='http://gdata.youtube.com/schemas/2007#video'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='80&apos;s'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='1984'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='the'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='After'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Puttin&apos;'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Taco'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Eighties'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Eight'/><category scheme='http://gdata.youtube.com/schemas/2007/categories.cat' term='Music' label='Music'/><title type='text'>Taco - Puttin' on the Ritz (Original Uncensored Version)</title><content type='text'>This is the original version of the video. The infamous &quot;blackface&quot; that was removed is shown here.</content><link rel='alternate' type='text/html' href='http://www.youtube.com/watch?v=OG3PnQ3tgzY'/><link rel='http://gdata.youtube.com/schemas/2007#video.responses' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/OG3PnQ3tgzY/responses'/><link rel='http://gdata.youtube.com/schemas/2007#video.related' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/OG3PnQ3tgzY/related'/><link rel='self' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/OG3PnQ3tgzY'/><author><name>ellocorod</name><uri>http://gdata.youtube.com/feeds/api/users/ellocorod</uri></author><media:group><media:title type='plain'>Taco - Puttin' on the Ritz (Original Uncensored Version)</media:title><media:description type='plain'>This is the original version of the video. The infamous &quot;blackface&quot; that was removed is shown here.</media:description><media:keywords>1984, 80's, After, Eight, Eighties, Ockerse, on, Puttin', Ritz, Taco, the</media:keywords><yt:duration seconds='273'/><media:category label='Music' scheme='http://gdata.youtube.com/schemas/2007/categories.cat'>Music</media:category><media:content url='http://www.youtube.com/v/OG3PnQ3tgzY' type='application/x-shockwave-flash' medium='video' isDefault='true' expression='full' duration='273' yt:format='5'/><media:content url='rtsp://rtsp2.youtube.com/ChoLENy73wIaEQk2g-0Nnc9tOBMYDSANFEgGDA==/0/0/0/video.3gp' type='video/3gpp' medium='video' expression='full' duration='273' yt:format='1'/><media:content url='rtsp://rtsp2.youtube.com/ChoLENy73wIaEQk2g-0Nnc9tOBMYESARFEgGDA==/0/0/0/video.3gp' type='video/3gpp' medium='video' expression='full' duration='273' yt:format='6'/><media:player url='http://www.youtube.com/watch?v=OG3PnQ3tgzY'/><media:thumbnail url='http://img.youtube.com/vi/OG3PnQ3tgzY/2.jpg' height='97' width='130' time='00:02:16.500'/><media:thumbnail url='http://img.youtube.com/vi/OG3PnQ3tgzY/1.jpg' height='97' width='130' time='00:01:08.250'/><media:thumbnail url='http://img.youtube.com/vi/OG3PnQ3tgzY/3.jpg' height='97' width='130' time='00:03:24.750'/><media:thumbnail url='http://img.youtube.com/vi/OG3PnQ3tgzY/0.jpg' height='240' width='320' time='00:02:16.500'/></media:group><yt:statistics viewCount='200446'/><gd:rating min='1' max='5' numRaters='786' average='4.74'/><gd:comments><gd:feedLink href='http://gdata.youtube.com/feeds/api/videos/OG3PnQ3tgzY/comments' countHint='0'/></gd:comments></entry><entry><id>http://gdata.youtube.com/feeds/api/videos/OG3PnQ3tgzY</id><published>2007-05-17T14:21:39.000-07:00</published><updated>2007-05-17T14:21:39.000-07:00</updated><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Ockerse'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='on'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Ritz'/><category scheme='http://schemas.google.com/g/2005#kind' term='http://gdata.youtube.com/schemas/2007#video'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='80&apos;s'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='1984'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='the'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='After'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Puttin&apos;'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Taco'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Eighties'/><category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='Eight'/><category scheme='http://gdata.youtube.com/schemas/2007/categories.cat' term='Music' label='Music'/><title type='text'>Taco - Puttin' on the Ritz (Original Uncensored Version)</title><content type='text'>This is the original version of the video. The infamous &quot;blackface&quot; that was removed is shown here.</content><link rel='alternate' type='text/html' href='http://www.youtube.com/watch?v=OG3PnQ3tgzY'/><link rel='http://gdata.youtube.com/schemas/2007#video.responses' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/OG3PnQ3tgzY/responses'/><link rel='http://gdata.youtube.com/schemas/2007#video.related' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/OG3PnQ3tgzY/related'/><link rel='self' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/OG3PnQ3tgzY'/><author><name>ellocorod</name><uri>http://gdata.youtube.com/feeds/api/users/ellocorod</uri></author><media:group><media:title type='plain'>Taco - Puttin' on the Ritz (Original Uncensored Version)</media:title><media:description type='plain'>This is the original version of the video. The infamous &quot;blackface&quot; that was removed is shown here.</media:description><media:keywords>1984, 80's, After, Eight, Eighties, Ockerse, on, Puttin', Ritz, Taco, the</media:keywords><yt:duration seconds='273'/><media:category label='Music' scheme='http://gdata.youtube.com/schemas/2007/categories.cat'>Music</media:category><media:content url='http://www.youtube.com/v/OG3PnQ3tgzY' type='application/x-shockwave-flash' medium='video' isDefault='true' expression='full' duration='273' yt:format='5'/><media:content url='rtsp://rtsp2.youtube.com/ChoLENy73wIaEQk2g-0Nnc9tOBMYDSANFEgGDA==/0/0/0/video.3gp' type='video/3gpp' medium='video' expression='full' duration='273' yt:format='1'/><media:content url='rtsp://rtsp2.youtube.com/ChoLENy73wIaEQk2g-0Nnc9tOBMYESARFEgGDA==/0/0/0/video.3gp' type='video/3gpp' medium='video' expression='full' duration='273' yt:format='6'/><media:player url='http://www.youtube.com/watch?v=OG3PnQ3tgzY'/><media:thumbnail url='http://img.youtube.com/vi/OG3PnQ3tgzY/2.jpg' height='97' width='130' time='00:02:16.500'/><media:thumbnail url='http://img.youtube.com/vi/OG3PnQ3tgzY/1.jpg' height='97' width='130' time='00:01:08.250'/><media:thumbnail url='http://img.youtube.com/vi/OG3PnQ3tgzY/3.jpg' height='97' width='130' time='00:03:24.750'/><media:thumbnail url='http://img.youtube.com/vi/OG3PnQ3tgzY/0.jpg' height='240' width='320' time='00:02:16.500'/></media:group><yt:statistics viewCount='200446'/><gd:rating min='1' max='5' numRaters='786' average='4.74'/><gd:comments><gd:feedLink href='http://gdata.youtube.com/feeds/api/videos/OG3PnQ3tgzY/comments' countHint='0'/></gd:comments></entry></feed>";
		if(contents == null) { return null; /* failed to get page */ }
		/* total number of results */
		var i = nIndex;
		var o_contents = contents;
		while(contents.indexOf("<entry>") >= 0) {
//		while(pattern.exec( contents ) != null) {
			if((start_index <= i) && (i < (start_index + length))) {
				contents = contents.substring(contents.indexOf("<entry>"));
				var e_end = contents.indexOf("</entry>");
				var item = contents.substring(0, e_end+8);
				contents = contents.substring(item.length);
//				var item = RegExp.$1;	// <entry>～</entry> video information
				info = new Object();

				// Title
				if(item.match( /<title type=\'text\'>([^<]*)<\/title>/ )) {
					info.Title = RegExp.$1;
				}
				// Description
				if(item.match( /<content type=\'text\'>([^<]*)<\/content>/ )) {
					info.Description = RegExp.$1;
				}
				// Thumbnail
				if(item.match( /<media:thumbnail url='([^']*)'[^>]*>/ )) {
					info.ThumbnailURL = RegExp.$1;
				}
				// URL
				if(item.match( /<media:player url='([^']*)'[^>]*>/ )) {
					info.URL = RegExp.$1;
				}
				// view count
				if(item.match( /<yt:statistics viewCount='([^']*)'[^>]*>/ )) {
					info.ViewCount = RegExp.$1 - 0;
				}
				// Length
				if(item.match( /<yt:duration seconds='([^']*)'[^>]*>/ )) {
					info.LengthSeconds = RegExp.$1 - 0;
				}
				// Rating / rating count
				if(item.match( /<gd:rating .* numRaters='([^']*)' average='([^']*)'[^>]*>/ )) {
					info.RatingCount = RegExp.$1 - 0;
					info.RatingAvg = RegExp.$2 - 0;
				}
				info.attr = 7;
				result.VideoInfo.push( info );
			}
			i++;
		}
		nIndex = nIndex + 24;
		nPage++;
	}
	if(result.VideoInfo.length) {
		result.end   = result.start + (result.VideoInfo.length - 1);
	} else {
		result.total = 0;	// no results.
	}
	if(o_contents.match( /<openSearch:totalResults>\d*?<\openSearch:totalResults>/ )) {
		result.total = RegExp.$1 - 0;
	}
	return result;
}

// Service metadata
var YouTube = new Object();
YouTube.Name          = "YouTube";			/* Service name */
YouTube.Description   = "YouTube";			/* Service description */
YouTube.SearchDesc    = "YouTube";			/* Service search screen description */
YouTube.SearchOSKMode = 1;						/* OSK mode 1: English only  2: Japanese */
YouTube.CheckURL      = YouTube_CheckURL;	/* CheckURL function */
YouTube.GetURL        = YouTube_GetURL;		/* GetURL function */
YouTube.Search        = YouTube_Search;		/* Search function */

// add YouTube to site list
SiteList.push( YouTube );
